@@ -4,7 +4,7 @@ import numpy as np
 import copy
 
 from .util_ import procrustes, print_log, nearest_neighbors, sparse_matrix, lexargmax
-from .global_reg_ import procrustes_init, spectral_alignment, sdp_alignment, procrustes_final, rgd_final, compute_alignment_err
+from .global_reg_ import procrustes_init, spectral_alignment, sdp_alignment, procrustes_final, rgd_final, gpm_final, compute_alignment_err
 
 from scipy.linalg import svdvals
 from scipy.sparse.csgraph import minimum_spanning_tree, breadth_first_order
@@ -239,12 +239,25 @@ class GlobalViews:
     
     def vis_embedding_(self, d, intermed_param, C, Utilde,
                       n_Utilde_Utilde, global_opts, vis,
-                      vis_opts, title='', color_of_pts_on_tear=None):
+                      vis_opts, title='', color_of_pts_on_tear=None,
+                      contrib_of_view=None):
         M,n = Utilde.shape
         y = np.zeros((n,d))
         for s in range(M):
-            C_s = C[s,:].indices
-            y[C_s,:] = intermed_param.eval_({'view_index': s, 'data_mask': C_s})
+            if global_opts['to_tear']:
+                if contrib_of_view is None:
+                    C_s = C[s,:].indices
+                else:
+                    C_s = contrib_of_view[s,:].indices
+                y[C_s,:] += intermed_param.eval_({'view_index': s, 'data_mask': C_s})
+            else:
+                Utilde_s = Utilde[s,:].indices
+                y[Utilde_s,:] += intermed_param.eval_({'view_index': s, 'data_mask': Utilde_s})
+        
+        if global_opts['to_tear'] and (contrib_of_view is not None):
+            y = y/np.asarray(contrib_of_view.sum(0).T)
+        elif not global_opts['to_tear']:
+            y = y/np.asarray(Utilde.sum(0).T)
 
         if global_opts['color_tear']:
             if (color_of_pts_on_tear is None) and global_opts['to_tear']:
@@ -355,7 +368,7 @@ class GlobalViews:
         self.tracker['init_computed_at'] = time.time()
         if global_opts['compute_error']:
             self.log('Computing error.')
-            err = compute_alignment_err(d, Utilde, intermed_param)
+            err = compute_alignment_err(d, Utilde, intermed_param, Utilde.count_nonzero())
             self.log('Alignment error: %0.3f' % err, log_time=True)
             self.tracker['init_err'] = err
         
@@ -425,74 +438,79 @@ class GlobalViews:
             self.log('Refining with ' + refine_algo + ' algorithm for ' + str(max_iter1) + ' iterations.')
             self.log('Refinement iteration: %d' % it0, log_time=True)
             
+            if global_opts['to_tear']:
+                # Compute which points contribute to which views
+                # IOW, compute correspondence between views and
+                # points. Since to_tear is True, this is not
+                # same as Utilde.
+                cov_col = []
+                cov_row = []
+                ZZ = n_Utilde_Utilde.multiply(n_Utildeg_Utildeg)
+                ZZ.eliminate_zeros()
+                for i in range(n_clusters):
+                    seq = seq_of_intermed_views_in_cluster[i]
+                    rho = parents_of_intermed_views_in_cluster[i]
+                    seq_set = set(seq)
+                    cov_col_ = C[seq[0],:].indices.tolist()
+                    cov_col += cov_col_
+                    cov_row += [seq[0]]*len(cov_col_)
+                    for m in range(1,seq.shape[0]):
+                        s = seq[m]
+                        Z_s = ZZ[s,:]
+                        Z_s = Z_s.indices.tolist()
+                        Z_s = list(seq_set.intersection(Z_s))
+                        if len(Z_s) == 0: # ideally this should not happen
+                            Z_s = parents_of_intermed_views_in_cluster[cluster_of_intermed_view[s]][s]
+                            Z_s = [Z_s]
+
+                        cov_s = Utilde[s,:].multiply(C[Z_s,:].sum(axis=0))
+                        cov_col_ = cov_s.nonzero()[1].tolist()
+                        cov_col += cov_col_
+                        cov_row += [s]*len(cov_col_)
+
+                contrib_of_view = csr_matrix((np.ones(len(cov_col), dtype=bool),
+                                              (cov_row, cov_col)),
+                                             shape=C.shape,
+                                             dtype = bool)
+                contrib_of_view += C
+            
             if refine_algo == 'procrustes':
                 y = procrustes_final(y, d, Utilde, C, intermed_param, n_Utilde_Utilde, n_Utildeg_Utildeg,
                                      seq_of_intermed_views_in_cluster, parents_of_intermed_views_in_cluster,
                                      cluster_of_intermed_view, global_opts)
                     
-            else:
-                if global_opts['to_tear']:
-                    # Compute which points contribute to which views
-                    # IOW, compute correspondence between views and
-                    # points. Since to_tear is True, this is not
-                    # same as Utilde.
-                    cov_col = []
-                    cov_row = []
-                    ZZ = n_Utilde_Utilde.multiply(n_Utildeg_Utildeg)
-                    ZZ.eliminate_zeros()
-                    for i in range(n_clusters):
-                        seq = seq_of_intermed_views_in_cluster[i]
-                        rho = parents_of_intermed_views_in_cluster[i]
-                        seq_set = set(seq)
-                        cov_col_ = C[seq[0],:].indices.tolist()
-                        cov_col += cov_col_
-                        cov_row += [seq[0]]*len(cov_col_)
-                        for m in range(1,seq.shape[0]):
-                            s = seq[m]
-                            Z_s = ZZ[s,:]
-                            Z_s = Z_s.indices.tolist()
-                            Z_s = list(seq_set.intersection(Z_s))
-                            if len(Z_s) == 0: # ideally this should not happen
-                                Z_s = parents_of_intermed_views_in_cluster[cluster_of_intermed_view[s]][s]
-                                Z_s = [Z_s]
-                            
-                            cov_s = Utilde[s,:].multiply(C[Z_s,:].sum(axis=0))
-                            cov_col_ = cov_s.nonzero()[1].tolist()
-                            cov_col += cov_col_
-                            cov_row += [s]*len(cov_col_)
-                    
-                    contrib_of_view = csr_matrix((np.ones(len(cov_col), dtype=bool),
-                                                  (cov_row, cov_col)),
-                                                 shape=C.shape,
-                                                 dtype = bool)
-                    contrib_of_view += C
-                            
-                if refine_algo == 'rgd':
-                    y = rgd_final(y, d, contrib_of_view, C, intermed_param,
-                                   n_Utilde_Utilde, n_Utildeg_Utildeg,
-                                   parents_of_intermed_views_in_cluster,
-                                   cluster_of_intermed_view,
-                                   global_opts)
-                elif refine_algo == 'spectral':
-                    y, y_2,\
-                    is_visited_view = spectral_alignment(y, is_visited_view, d, contrib_of_view,
-                                                         C, intermed_param, global_opts,
-                                                         seq_of_intermed_views_in_cluster)
-                elif refine_algo == 'sdp':
-                    y, y_2,\
-                    is_visited_view,\
-                    solver = sdp_alignment(y, is_visited_view, d, contrib_of_view,
-                                           C, intermed_param, global_opts,
-                                           seq_of_intermed_views_in_cluster,
-                                           solver=solver)
+            elif refine_algo == 'rgd':
+                y = rgd_final(y, d, contrib_of_view, C, intermed_param,
+                               n_Utilde_Utilde, n_Utildeg_Utildeg,
+                               parents_of_intermed_views_in_cluster,
+                               cluster_of_intermed_view,
+                               global_opts)
+            elif refine_algo == 'gpm':
+                y = gpm_final(y, d, contrib_of_view, C, intermed_param,
+                               n_Utilde_Utilde, n_Utildeg_Utildeg,
+                               parents_of_intermed_views_in_cluster,
+                               cluster_of_intermed_view,
+                               global_opts)
+            elif refine_algo == 'spectral':
+                y, y_2,\
+                is_visited_view = spectral_alignment(y, is_visited_view, d, contrib_of_view,
+                                                     C, intermed_param, global_opts,
+                                                     seq_of_intermed_views_in_cluster)
+            elif refine_algo == 'sdp':
+                y, y_2,\
+                is_visited_view,\
+                solver = sdp_alignment(y, is_visited_view, d, contrib_of_view,
+                                       C, intermed_param, global_opts,
+                                       seq_of_intermed_views_in_cluster,
+                                       solver=solver)
                 
             self.log('Done.', log_time=True)
             self.tracker['refine_iter_done_at'][it0] = time.time()
 
             if global_opts['compute_error'] or (it0 == max_iter0-1):
                 self.log('Computing error.')
-                err = compute_alignment_err(d, Utilde, intermed_param)
-                self.log('Alignment error: %0.3f' % err, log_time=True)
+                err = compute_alignment_err(d, contrib_of_view, intermed_param, Utilde.count_nonzero())
+                self.log('Alignment error: %0.6f' % err, log_time=True)
                 self.tracker['refine_err_at_iter'][it0] = err
                 
             self.add_spacing_bw_clusters(d, seq_of_intermed_views_in_cluster,
@@ -517,6 +535,7 @@ class GlobalViews:
             _, y = self.vis_embedding_(d, intermed_param, C, Utilde,
                               n_Utilde_Utilde, global_opts, vis,
                               vis_opts, title='Iter_%d' % it0,
-                              color_of_pts_on_tear=color_of_pts_on_tear)
+                              color_of_pts_on_tear=color_of_pts_on_tear,
+                              contrib_of_view=contrib_of_view)
 
         return y , color_of_pts_on_tear
\ No newline at end of file
