@@ -180,7 +180,7 @@ def double_manifold_k_nn(data, ddX, k_nn, metric, n_proc=1):
     
 
 def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, gl_type='unnorm',
-                           N=100, no_gamma=False, Atilde_method='LDLE_1',
+                           N=100, scale_by='gamma', Atilde_method='LDLE_1',
                            alpha=1, max_iter=300, reg=0.,
                            p=0.99, tau=50, delta=0.9, to_postprocess= True,
                            pp_n_thresh=32):
@@ -212,9 +212,12 @@ def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, gl_type='unnorm
         Number of smallest non-trivial eigenvectors of the
         graph Laplacian to be used for the construction of
         local views.
-    no_gamma : bool
-               If True, the parameterization of local views
-               are not normalized.
+    scale_by : str
+               To scale the eigenvectors. Options
+               are 'none', 'gamma' or a positive real value. Default is
+               'gamma'. Using numerical value with gl_type as 'diffusion'
+               uses diffusion maps to construct local views where
+               the numeric value is used as the power of the eigenvalues.
     Atilde_method : str
                     Method to use for conmputing Atilde.
                     Options are 'LDLE_1' for finite element method,
@@ -246,7 +249,7 @@ def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, gl_type='unnorm
         value such as 32 leads to faster postprocessing.
     """
     return {'k_nn': k_nn, 'k_tune': k_tune, 'k': k,
-           'gl_type': gl_type, 'N': N, 'no_gamma': no_gamma,
+           'gl_type': gl_type, 'N': N, 'scale_by': scale_by,
            'Atilde_method': Atilde_method, 'p': p, 'tau': tau, 'delta': delta,
            'alpha': alpha, 'max_iter': max_iter, 'reg': reg, 
            'to_postprocess': to_postprocess, 'algo': algo,
@@ -330,6 +333,7 @@ def get_default_global_opts(main_algo='LDLE', to_tear=True, nu=3, max_iter=20, c
                        Options are 'gpa' for Generalized Procustes Analysis
                        (GPA) based alignment, 'rgd' for Riemannian gradient descent
                        based alignment, 'spectral' for spectral alignment,
+                       'gpm' for generalized power method based alignment,
                        'sdp' for semi-definite programming based alignment. Note that
                        sdp based alignment is very slow.
     max_internal_iter : int
@@ -477,6 +481,7 @@ class LDLE:
         # The variables created during the fit
         self.X = None
         self.d_e = None
+        self.scale = None
         self.ddX = None
         self.neigh_dist = None
         self.neigh_ind = None
@@ -529,6 +534,7 @@ class LDLE:
             self.log('Done.', log_time=True)
         
         # Construct a sparse d_e matrix based on neigh_ind and neigh_dist
+        self.scale = np.min(neigh_dist[neigh_dist > 0])
         d_e = sparse_matrix(neigh_ind, neigh_dist)
         d_e = d_e.maximum(d_e.transpose())
 
