@@ -19,6 +19,13 @@ from matplotlib.offsetbox import OffsetImage, AnnotationBbox
 from matplotlib.cbook import get_sample_data
 from scipy.spatial.distance import pdist, squareform
 
+def combine_cmaps(zeta, U_k):
+    min_zeta = np.min(zeta)
+    max_zeta = np.max(zeta)
+    c1 = plt.cm.jet((zeta - min_zeta)/(max_zeta-min_zeta))
+    c1[U_k,:-1] = 0
+    return c1
+
 def eval_param(phi, Psi_gamma, Psi_i, k, mask, beta=None, T=None, v=None):
     if beta is None:
         return Psi_gamma[k,:][np.newaxis,:] * phi[np.ix_(mask,Psi_i[k,:])]
@@ -173,10 +180,10 @@ class Visualize:
             set_axes_equal(ax)
         plt.title(title)
         plt.tight_layout()
-        plt.axis('off')
-        plt.show()
+        #plt.axis('off')
         if self.save_dir:
             plt.savefig(self.save_dir+'/' + title + '.png') 
+        plt.show()
         
     def eigenvalues(self, lmbda, figsize=None):
         fig = plt.figure(figsize=figsize)
@@ -187,9 +194,9 @@ class Visualize:
         plt.ylabel('$\lambda_i$')
         plt.xlabel('i')
         plt.title('Eigenvalues')
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/eigenvalues.png') 
+        plt.show()
         
     def gamma(self, X, gamma, i, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -207,11 +214,11 @@ class Visualize:
             set_axes_equal(ax)
             fig.colorbar(p)
         plt.title('$\gamma_{%d}$'%i)
-        plt.show()
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/gamma'):
                 os.makedirs(self.save_dir+'/gamma')
             plt.savefig(self.save_dir+'/gamma/'+str(i)+'.png') 
+        plt.show()
     
     def eigenvector(self, X, phi, i, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -229,11 +236,11 @@ class Visualize:
             set_axes_equal(ax)
             fig.colorbar(p)
         plt.title('$\phi_{%d}$'%i)
-        plt.show()
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/eigvecs'):
                 os.makedirs(self.save_dir+'/eigvecs')
             plt.savefig(self.save_dir+'/eigvecs/'+str(i)+'.png') 
+        plt.show()
     
     def grad_phi(self, X, phi, grad_phi, i, prop=0.01, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -268,11 +275,12 @@ class Visualize:
             p = ax.quiver(X[mask,0], X[mask,1], X[mask,2], grad_phi[mask,i,0], grad_phi[mask,i,1], grad_phi[mask,i,2])
             set_axes_equal(ax)
             plt.title('$\\nabla\phi_{%d}$'%i)
-        plt.show()
+        
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/grad_phi'):
                 os.makedirs(self.save_dir+'/grad_phi')
             plt.savefig(self.save_dir+'/grad_phi/'+str(i)+'.png') 
+        plt.show()
     
     def Atilde(self, X, phi, i, j, Atilde, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -313,11 +321,12 @@ class Visualize:
             set_axes_equal(ax)
             fig.colorbar(p, ax=ax)
             plt.title('$\widetilde{A}_{:%d%d}$'%(i,j))
-        plt.show()
+        
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/Atilde'):
                 os.makedirs(self.save_dir+'/Atilde')
             plt.savefig(self.save_dir+'/Atilde/'+str(i)+'_'+str(j)+'.png') 
+        plt.show()
     
     def n_eigvecs_w_grad_lt(self, X, Atilde, thresh_prctile=None, figsize=(16,8), s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -388,12 +397,12 @@ class Visualize:
                 if cb is not None:
                     cb.remove()
                 cb = fig.colorbar(p, ax=ax)
-                ax.set_title('$n_k = \sum_{i}\widetilde{A}_{kii} < %f$'% thresh)
-            plt.show()    
+                ax.set_title('$n_k = \sum_{i}\widetilde{A}_{kii} < %f$'% thresh)   
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/n_eigvecs_w_grad_lt'):
                     os.makedirs(self.save_dir+'/n_eigvecs_w_grad_lt')
                 plt.savefig(self.save_dir+'/n_eigvecs_w_grad_lt/'+str(thresh)+'.png')
+            plt.show()
             
             if thresh_prctile is not None:
                 break
@@ -418,16 +427,16 @@ class Visualize:
             set_axes_equal(ax)
             fig.colorbar(p)
         plt.title(title)
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/'+title+'.png') 
+        plt.show()
     
     def distortion_boxplot(self, zeta, title, figsize=None):
         fig = plt.figure(figsize=figsize)
         plt.boxplot([zeta],labels=[title], notch=True, patch_artist=True)
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/box_'+title+'.png') 
+        plt.show()
     
     def dX(self, X, ddX, title, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -454,10 +463,10 @@ class Visualize:
             p = ax.scatter(X[:,0], X[:,1], X[:,2], s=s, c=ddX==0, cmap='jet')
             set_axes_equal(ax)
             fig.colorbar(p, ax=ax)
-            ax.set_title('dX')
-        plt.show()    
+            ax.set_title('dX') 
         if self.save_dir:
             plt.savefig(self.save_dir+'/'+title+'.png') 
+        plt.show()
     
     def intrinsic_dim(self, X, chi, figsize=None, s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -505,9 +514,9 @@ class Visualize:
             fig.colorbar(p)
             ax.set_title('$\\phi_{i_2}$')
         fig.tight_layout()
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/chosen_eigvecs_for_local_views.png') 
+        plt.show()
     
     def chosen_eigevec_inds_for_intermediate_views(self, X, Psitilde_i, c, figsize=(16,8), s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -544,9 +553,9 @@ class Visualize:
             fig.colorbar(p)
             ax.set_title('$\\phi_{i_2}$')
         fig.tight_layout()
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/chosen_eigvecs_for_intermediate_views.png') 
+        plt.show()
     
     def local_views(self, X, local_param, U, gamma, Atilde, k=None, save_subdir='', figsize=(15,10), s=20):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
@@ -613,9 +622,10 @@ class Visualize:
             ax[0].cla()
             cb[0].remove()
             if is_3d_data:
-                p = ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s*(1-U_k), c=zeta, cmap='jet')
-                ax[0].scatter(X[U_k,0], X[U_k,1], X[U_k,2], s=s, c='k')
+                p=ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s, c=zeta, cmap='jet')
+                ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s, c=combine_cmaps(zeta, U_k))
                 set_axes_equal(ax[0])
+                
             else:
                 p = ax[0].scatter(X[:,0], X[:,1], s=s*(1-U_k), c=zeta, cmap='jet')
                 ax[0].scatter(X[U_k,0], X[U_k,1], s=s, c='k')
@@ -646,8 +656,8 @@ class Visualize:
                 if not first_plot:
                     cb[j+1].remove()
                 if is_3d_data:
-                    p = ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s*(1-U_k), c=local_param.phi[:n,i_s], cmap='jet')
-                    ax[j+1].scatter(X[U_k,0], X[U_k,1], X[U_k,2], s=s, c='k')
+                    p=ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s, c=local_param.phi[:n,i_s], cmap='jet')
+                    ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s, c=combine_cmaps(local_param.phi[:n,i_s], U_k))
                     set_axes_equal(ax[j+1])
                 else:
                     p = ax[j+1].scatter(X[:,0], X[:,1], s=s*(1-U_k), c=local_param.phi[:n,i_s], cmap='jet')
@@ -687,11 +697,11 @@ class Visualize:
             fig.tight_layout()    
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_views/'+save_subdir):
                     os.makedirs(self.save_dir+'/local_views/'+save_subdir)
                 plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.png')
+            plt.show()
             
             first_plot = False
             if not k_not_available:
@@ -810,18 +820,18 @@ class Visualize:
             fig.tight_layout()    
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_views/'+save_subdir):
                     os.makedirs(self.save_dir+'/local_views/'+save_subdir)
                 plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.png')
+            plt.show()
             
             first_plot = False
             if not k_not_available:
                 break
     
     def intermediate_views(self, X, phi, Utilde, gamma, Atilde, Psitilde_gamma,
-              Psitilde_i, zetatilde, c, k=None, figsize=(15,10), s=20):
+              Psitilde_i, zetatilde, c, k=None, figsize=(15,10), s=20, save_subdir=''):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
         is_3d_data = X.shape[1] == 3
         n,N = phi.shape
@@ -889,8 +899,8 @@ class Visualize:
             ax[0].cla()
             cb[0].remove()
             if is_3d_data:
-                p = ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s*(1-Utilde_m), c=zeta, cmap='jet')
-                ax[0].scatter(X[Utilde_m,0], X[Utilde_m,1], X[Utilde_m,2], s=s, c='k')
+                p=ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s, c=zeta, cmap='jet')
+                ax[0].scatter(X[:,0], X[:,1], X[:,2], s=s, c=combine_cmaps(zeta, Utilde_m))
                 set_axes_equal(ax[0])
             else:
                 p = ax[0].scatter(X[:,0], X[:,1], s=s*(1-Utilde_m), c=zeta, cmap='jet')
@@ -920,8 +930,8 @@ class Visualize:
                 if not first_plot:
                     cb[j+1].remove()
                 if is_3d_data:
-                    p = ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s*(1-Utilde_m), c=phi[:,i_s], cmap='jet')
-                    ax[j+1].scatter(X[Utilde_m,0], X[Utilde_m,1], X[Utilde_m,2], s=s, c='k')
+                    p=ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s, c=phi[:,i_s], cmap='jet')
+                    ax[j+1].scatter(X[:,0], X[:,1], X[:,2], s=s, c=combine_cmaps(phi[:,i_s], Utilde_m))
                     set_axes_equal(ax[j+1])
                 else:
                     p = ax[j+1].scatter(X[:,0], X[:,1], s=s*(1-Utilde_m), c=phi[:,i_s], cmap='jet')
@@ -960,11 +970,11 @@ class Visualize:
                 
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
-                if not os.path.isdir(self.save_dir+'/intermediate_views'):
-                    os.makedirs(self.save_dir+'/intermediate_views')
-                plt.savefig(self.save_dir+'/intermediate_views/'+str(m)+'.png') 
+                if not os.path.isdir(self.save_dir+'/intermediate_views/'+save_subdir):
+                    os.makedirs(self.save_dir+'/intermediate_views/'+save_subdir)
+                plt.savefig(self.save_dir+'/intermediate_views/'+save_subdir+'/'+str(m)+'.png') 
+            plt.show()
             
             first_plot = False
             if not k_not_available:
@@ -1074,11 +1084,11 @@ class Visualize:
                       \\gamma_{ki_2}\\sqrt{\\widetilde{A}_{ki_2i_2}}+1)$')
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_high_low_distortion/'):
                     os.makedirs(self.save_dir+'/local_high_low_distortion/')
                 plt.savefig(self.save_dir+'/local_high_low_distortion/thresh='+str(thresh)+'.png') 
+            plt.show()
             
         
     def compare_intermediate_high_low_distortion(self, X, Atilde, Psitilde_gamma, 
@@ -1192,11 +1202,11 @@ class Visualize:
                       \\gamma_{ki_2}\\sqrt{\\widetilde{A}_{ki_2i_2}}+1)$')
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/intermediate_high_low_distortion'):
                     os.makedirs(self.save_dir+'/intermediate_high_low_distortion')
                 plt.savefig(self.save_dir+'/intermediate_high_low_distortion/thresh='+str(thresh)+'.png') 
+            plt.show()
     
     def seq_of_intermediate_views(self, X, c, seq, rho, Utilde, figsize=None, s=20, cmap='jet'):
         seq = np.copy(seq)
@@ -1238,9 +1248,9 @@ class Visualize:
             set_axes_equal(ax)
             fig.colorbar(p)
         plt.title('Views colored in the sequence they are visited')
-        plt.show()
         if self.save_dir:
             plt.savefig(self.save_dir+'/seq_in_which_views_are_visited.png') 
+        plt.show()
     
     def global_embedding(self, y, labels, cmap0, color_of_pts_on_tear=None, cmap1=None,
                          title=None, figsize=None, s=30, set_title=False):
@@ -1305,11 +1315,11 @@ class Visualize:
         plt.margins(0)
         plt.gca().xaxis.set_major_locator(plt.NullLocator())
         plt.gca().yaxis.set_major_locator(plt.NullLocator())
-        plt.show()
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/ge'):
                 os.makedirs(self.save_dir+'/ge')
             plt.savefig(self.save_dir+'/ge/'+str(title)+'.png', bbox_inches = 'tight',pad_inches = 0)
+        plt.show()
     
     def global_embedding_images(self, X, img_shape, y, labels, cmap0, color_of_pts_on_tear=None, cmap1=None,
                          title=None, figsize=None, s=30, zoom=1, offset_ratio=0.2,w_ratio=0.0025):
@@ -1408,11 +1418,11 @@ class Visualize:
             ax.set_title('Duble click to choose a point. Press button to exit')
             fig.canvas.draw()
             fig.canvas.flush_events()
-            plt.show()
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/ge_img'):
                     os.makedirs(self.save_dir+'/ge_img')
                 plt.savefig(self.save_dir+'/ge_img/'+str(title)+'.png')
+            plt.show()
     
     def global_embedding_images_v2(self, X, img_shape, y, labels, cmap0, color_of_pts_on_tear=None, cmap1=None,
                          title='images', offset_ratio=0.3, zoom=1, nx=8, ny=8, v_ratio=0.8, w_ratio=0.005,
